from .api_helper import ApiHelper


class Authorization(ApiHelper):
    # AUTHENTICATION CONSTANTS
    CLIENT_CREDENTIALS_PARAM = 'client_credentials'
    CLIENT_ID_PARAM = 'client_id'
    CLIENT_SECRET_PARAM = 'client_secret'
    GRANT_TYPE_PARAM = 'grant_type'
    SCOPE_PARAM = 'scope'

    OAUTH_URL = '/oauth2/token'

    def get_access_token(self):
        """Generates an access token to be used for all requests.

        Args:
            scopes (str): Specific scopes the returned token should have.

        Returns:
            (dict): containing the access_token key.
        """
        data = {
            self.GRANT_TYPE_PARAM: self.CLIENT_CREDENTIALS_PARAM,
            self.CLIENT_ID_PARAM: self.client_id,
            self.CLIENT_SECRET_PARAM: self.client_secret,
            self.SCOPE_PARAM: self.scopes,
        }

        return self._request(self.OAUTH_URL, data=data, is_oauth=True, method=self.HTTP_POST)


import requests

from insided import __version__


class InsidedResponse(object):
    def __init__(self, data, status_code, response):
        self.data = data
        self.status_code = status_code
        self.response = response


class ApiHelper(object):
    # HTTP METHOD CONSTANTS
    HTTP_DELETE = 'delete'
    HTTP_GET = 'get'
    HTTP_POST = 'post'
    HTTP_PUT = 'put'

    DEFAULT_API_SCOPES = 'write read'
    PAGE_PARAM = 'page'
    PAGE_SIZE_PARAM = 'pageSize'
    FILTER_PARAM = 'filter'

    # URL CONSTANTS
    BASE_URL = 'https://api2-us-west-2.insided.com'

    def __init__(self, client_id=None, client_secret=None, access_token=None, user_agent=None, scopes=None):
        self.client_id = client_id
        self.client_secret = client_secret
        self.access_token = access_token
        self.scopes = scopes or self.DEFAULT_API_SCOPES
        self.user_agent = user_agent or 'InsidedPythonSDK/{}'.format(__version__)

    def _request(self, path, params=None, headers={}, data=None, json=None, is_oauth=False, method=HTTP_GET):
        """Makes an HTTP request to the insided api.

        Args:
            path (str): The ending pathname.
            params (dict): Query parameters.
            headers (dict): HTTP Headers for the request.
            data (dict): Information to be passed into the body.
            json (dict): Information to be passed into the body as json.
            is_oauth (bool): Authentication request or not.
            method (str): HTTP method.

        Returns:
            (InsidedResponse): Information from the HTTP Response
        """
        url = '{}{}'.format(self.BASE_URL, path)
        headers = self._build_headers(headers, is_oauth)
        response = getattr(requests, method.lower())(
            url=url,
            params=params,
            headers=headers,
            data=data,
            json=json
        )
        return self._build_response(response)

    def _build_response(self, response):
        """Creates a response object.

        Args:
            response (Response): HTTP response from api.

        Returns:
            (InsidedResponse): Information containing the response.
        """
        if response.status_code in (202, 204,):
            data = response.text
        else:
            data = response.json()

        return InsidedResponse(
            data,
            response.status_code,
            response
        )

    def _build_headers(self, headers, is_oauth=False):
        """Dynamically generates headers based on specific request.

        Args:
            headers (dict):
            is_oauth (bool): True if we should use form encoding for oauth.

        Returns:
            (dict): Headers
        """
        headers['User-Agent'] = self.user_agent

        if is_oauth:
            headers.update({
                'Content-Type': 'application/x-www-form-urlencoded'
            })
        else:
            headers.update({
                'Authorization': 'Bearer {}'.format(self.access_token),
            })

        return headers





import sys

from insided import __version__
from .authorization import Authorization
from .gamification import Gamification
from .users import Users
from .webhooks import Webhooks


class Insided(object):
    """The API Wrapper around the InSided API

    Args:
        client_id (str): Client ID for Insided API.
        client_secret (str): Client Secret for Insided API.
        access_token (str): Bearer token to make API calls with.
        client_id (str): User agent for HTTP requests.
    """
    def __init__(self, client_id=None, client_secret=None, access_token=None, user_agent=None, scopes=None):
        self.client_id = client_id
        self.client_secret = client_secret
        self.access_token = access_token
        self.scopes = scopes
        self.user_agent = user_agent or 'InsidedPythonSDK/{}'.format(__version__)

    def __getattr__(self, item):
        return InsidedWrapper(item, self)


class InsidedWrapper(object):
    def __init__(self, resource_class, api, *args, **kwargs):
        """An API Wrapper to dynamically load the class and method"""
        if isinstance(resource_class, str):
            self.resource_class = self.str_to_class(resource_class, api)
        else:
            self.resource_class = resource_class

    def __getattr__(self, item):
        """Overwrite to make us dynamically call the called class and it's method automatically"""
        return lambda *args, **kwargs: getattr(self.resource_class, item)(*args, **kwargs)

    @classmethod
    def str_to_class(cls, str, api):
        """Transforms a string class name into a class object.
        Assumes that the class is already loaded.
        Args:
            str (str): Name of a class.
            api (obj): Insided api object.
        Returns:
            (dict): Insided API Response
        """
        return getattr(sys.modules[__name__], str)(
            client_id=api.client_id,
            client_secret=api.client_secret,
            access_token=api.access_token,
            user_agent=api.user_agent
        )







from .api_helper import ApiHelper


class Users(ApiHelper):
    USER_URL = '/user'
    USER_CREATE_URL = '{}/register'.format(USER_URL)
    USER_ID_URL = USER_URL + '/{}'
    USER_ACTIVITY_URL = '{}/activity'.format(USER_URL)
    USER_ID_DELETE_URL = USER_URL + '/{}/erase'
    USER_ROLE_ADD_URL = USER_URL + '/{}/role'
    USER_FIELD_VALUE_URL = USER_URL +  '/{}/{}'
    USER_ID_FIELD_VALUE_URL = USER_ID_URL + '/{}/{}'
    USER_DELETE_CUSTOM_FIELD_URL = USER_URL + '/{}/profile_field/{}'
    USER_REMOTE_LOGOUT_URL = '/remotelogout/{}/{}'
    USER_BADGE_URL = USER_ID_URL + '/badge/{}'

    GAMIFICATION_BADGES_URL = '/gamification/badges'
    GAMIFICATION_BADGE_ID_URL = GAMIFICATION_BADGES_URL + '/{}'

    DEFAULT_STARTING_PAGE = 1
    DEFAULT_PAGE_SIZE = 10

    def get_users(self, page=DEFAULT_STARTING_PAGE, page_size=DEFAULT_PAGE_SIZE):
        """Fetches a list of all users.

        Args:
            page (int): Page number pagination starts from.
            page_size (int): Number of results returned in paginated list.

        Returns:
            (InsidedResponse): A list of users.
        """
        params = {
            self.PAGE_PARAM: page,
            self.PAGE_SIZE_PARAM: page_size
        }

        return self._request(self.USER_URL, params=params)

    def create_user(self, email, password, username=None):
        """Generates a new User in Insided.

        Args:
            email (str): User's email address.
            password (str): User's password.
            username (str): User's username.

        Returns:
            (InsidedResponse): Information about the created user.
        """
        data = {
            'data': {
                'email': email,
                'username': username or email,
                'password': password
            }
        }
        return self._request(self.USER_CREATE_URL, json=data, method=self.HTTP_POST)

    def get_user_by_id(self, user_id):
        """Fetches a user by their ID.

        Args:
            user_id (int): Unique User ID.

        Returns:
            (InsidedResponse): Information about the requested User.
        """
        return self._request(self.USER_ID_URL.format(user_id))

    def get_user_activity(self, page=DEFAULT_STARTING_PAGE, page_size=DEFAULT_PAGE_SIZE, api_filter=None):
        """Fetches all users activities or a single user's activities in public categories.

        Args:
            page (int): Page number pagination starts from.
            page_size (int): Number of results returned in paginated list.
            api_filter (array[array]): Filter based on subset of defined events.

        Returns:
            (InsidedResponse): User activity.
        """
        params = {
            self.PAGE_PARAM: page,
            self.PAGE_SIZE_PARAM: page_size
        }
        if api_filter:
            params[self.FILTER_PARAM] = api_filter

        return self._request(self.USER_ACTIVITY_URL, params=params)

    def delete_user(self, user_id):
        """Removes a user.

        Args:
            user_id (int): Unique User ID.

        Returns:
            (InsidedResponse): Information about the removed user.
        """
        return self._request(self.USER_ID_DELETE_URL.format(user_id), method=self.HTTP_DELETE)

    def add_user_role(self, user_id, role=[]):
        """Adds a new role to a user.

        Args:
            user_id (int): Unique User ID.
            role (list): Roles to be added to the user.

        Returns:
            (InsidedResponse)
        """
        data = {
            'data': role
        }
        return self._request(self.USER_ROLE_ADD_URL.format(user_id), json=data)


    def get_user_by_field(self, field, value):
        """Get's a user by field.

        Args:
            field (str): The field to look for.
            value (str): The value to look for.

        Returns:
            (InsidedResponse): Returns a Json User.
        """
        return self._request(self.USER_FIELD_VALUE_URL.format(field, value))

    def update_user_field_profile(self, user_id, field, value):
        """Updates a user field.

        Args:
            user_id (int): Unique User ID.
            field (str): The field that will be updated.
            value (str):The new value (url encoded).

        Returns:
            (InsidedResponse): Returns a Json User.
        """
        return self._request(self.USER_ID_FIELD_VALUE_URL.format(user_id, field, value), method=self.HTTP_PUT)

    def delete_user_profile_field(self, user_id, field):
        """Deletes a users profile field.

        Args:
            user_id (int): Unique User ID.
            field (str): The field that will be deleted.

        Returns:
            (InsidedResponse): Returns a Json User.
        """
        return self._request(self.USER_DELETE_CUSTOM_FIELD_URL.format(user_id, field), method=self.HTTP_DELETE)

    def user_logout(self, field, value):
        """Generates a Remote logout for a User.

        Args:
            field (str): The field to look for.
            value (str): The value to look for.

        Returns:
            (InsidedResponse): Returns a Json User.
        """
        return self._request(self.USER_REMOTE_LOGOUT_URL.format(field, value), method=self.HTTP_DELETE)

    def update_user_badge(self, user_id, badge_id):
        """Award a badge identified by ID to the user identified by the user ID.

        Args:
            user_id (int): Unique User ID.
            badge_id (int): The ID of the badge to be awarded.

        Returns:
            (InsidedResponse): Empty if succcessful.
        """
        return self._request(self.USER_BADGE_URL.format(user_id, badge_id), method=self.HTTP_PUT)

    def remove_user_badge(self, user_id, badge_id):
        """Revoke badge from user.

        Args:
            user_id (int): Unique User ID.
            badge_id (int): The id of the badge to be revoked.

        Returns:
            (InsidedResponse): Empty if succcessful.
        """
        return self._request(self.USER_BADGE_URL.format(user_id, badge_id), method=self.HTTP_DELETE)

    def get_badges(self, page=DEFAULT_STARTING_PAGE, page_size=DEFAULT_PAGE_SIZE):
        """Gets a list of all badges.

        Args:
            page (int): Page number pagination starts from.
            page_size (int): Number of results returned in paginated list.

        Returns:
            (InsidedResponse): A list of badges.
        """
        params = {
            self.PAGE_PARAM: page,
            self.PAGE_SIZE_PARAM: page_size
        }
        return self._request(self.GAMIFICATION_BADGES_URL, params=params)

    def delete_badge(self, badge_id):
        """Permanently delete a badge identified by ID from community and revoke it from all users.

        Args:
            badge_id (int): The id of the badge to be removed.

        Returns:
            (InsidedResponse): Status code 304 if successful.
        """
        return self._request(self.GAMIFICATION_BADGE_ID_URL.format(badge_id), method=self.HTTP_DELETE)


from .api_helper import ApiHelper

class Webhooks(ApiHelper):
    WEBHOOKS_URL = '/v2/webhooks'
    WEBHOOK_SUBSCRIBE_URL = WEBHOOKS_URL + '/{}/subscriptions'

    def create_webhook(self, webhook_name, webhook_response_url):
        """Generates a webhook for an action.

        Available webhook_names found here: https://api2-us-west-2.insided.com/docs/#section/Webhooks/Available-events

        Args:
            webhook_name (str): Name of the actual event.
            webhook_response_url (str): Name of the url Insided should send a webhook too.

        Returns:

        """
        data = {
            'url': webhook_response_url,
            'username': self.client_id,
            'secret': self.client_secret
        }

        self._request(self.WEBHOOK_SUBSCRIBE_URL.format(webhook_name), json=data, method=self.HTTP_POST)

    def get_webhooks(self):
        """Returns a list of all subscribed webhooks."""
        return self._request(self.WEBHOOKS_URL)



from .api_helper import ApiHelper

class Gamification(ApiHelper):
    EXCLUDED_PARAM = 'excluded'

    LEADERBOARD_URL = '/leaderboard'
    LEADERBOARD_WEEKLY_URL = '{}/weekly'.format(LEADERBOARD_URL)
    POINTS_URL = '/points'
    POINTS_ASSIGN_URL = '/{}/assign'.format(POINTS_URL)

    DEFAULT_STARTING_PAGE = 1
    DEFAULT_PAGE_SIZE = 25

    def get_leaderboard(self, excluded, page=DEFAULT_STARTING_PAGE, page_size=DEFAULT_PAGE_SIZE):
        """Fetches a paginated list of users sorted by ascending order of all time points earned.

        Args:
            excluded (list): Muliple user roles can be provided with comma separated strings. Use roles.banned,roles.moderator for testing.
            page (int): Page number pagination starts from.
            page_size (int): Number of results returned in paginated list.

        Returns:
            (InsidedResponse): List of users.
        """
        params = {
            self.EXCLUDED_PARAM: excluded,
            self.PAGE_PARAM: page,
            self.PAGE_SIZE_PARAM: page_size
        }

        return self._request(self.LEADERBOARD_URL, params=params)


    def get_weekly_leaderbaord(self, excluded, page=DEFAULT_STARTING_PAGE, page_size=DEFAULT_PAGE_SIZE):
        """Fetches a paginated list of users sorted by ascending order of points earned this week.

        'This week' is a fixed 7 day time period (Monday-Sunday), rather than a rolling 7 day period.
        This is reset based on the value of timezone chosen when the point system is configured in the control environment.

        Args:
            excluded (list): Muliple user roles can be provided with comma separated strings. Use roles.banned,roles.moderator for testing.
            page (int): Page number pagination starts from.
            page_size (int): Number of results returned in paginated list.

        Returns:
            (InsidedResponse): List of users.
        """
        params = {
           self.EXCLUDED_PARAM: excluded,
           self.PAGE_PARAM: page,
           self.PAGE_SIZE_PARAM: page_size
        }

        return self._request(self.LEADERBOARD_WEEKLY_URL, params=params)

    def assign_points(self, user_id, points):
        """Assign points to a single user by UserId.

        Args:
            user_id (int): Unique User ID.
            points (int): Amount of points to assign.

        Returns:
            (InsidedResponse): 204 status code if successful.
        """
        data = {
            'user': user_id,
            'points': points
        }
        return self._request(self.POINTS_ASSIGN_URL, json=data, method=self.HTTP_POST)

    def get_points(self, user_ids, earned_from=None, earned_at=None):
        """Get points assigned to users in a timeframe.

        Args:
            user_ids (list): Unique User ID's.
            earned_from (str): Starting value for a timeframe. Uses beginning of unix time as default.
            earned_at (str): Ending value for a timeframe. Uses current time as a default.

        Returns:
            (InsidedResponse): List of users with total points assigned.
        """
        params = {
            'userId[]': user_ids,
            'earnedAt[from]': earned_from,
            'earnedAt[to]': earned_at
        }
        return self._request(self.POINTS_URL, params=params)


import os
from setuptools import setup, find_packages

VERSION = '1.0.1'

setup(
    name='insided',
    version=VERSION,

    packages=find_packages(),
    install_requires=['requests>=2.24.0'],

    url='https://github.com/remstone7/insided-python',
    download_url='',

    author='Remington Stone',
    author_email='remstone7@gmail.com',

    description='Connect Python applications with the InSided API',
    long_description=(
"""
This is  wrapper around the InSided community forum API.
"""),
    license='MIT',

    keywords=['insided', 'api',],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
    ],
)